/** Clinical6 Base Class */
export class Clinical6 {
  apiBaseUrl: string;
  appVersion: any;
  authToken: string;
  available: any;
  config: any;
  device: any;
  mobileApplicationKey: string;
  storage: Storage;
  user: User;

  delete(obj: any, cacheMode?: string): Promise<any>;
  fetch(url: string, method?: string, data?: any): Promise<any>;
  get(obj: any, options?: { cacheMode?: string, meta?: any }): Promise<any>;
  getChildren(parent: any, child?: any, options?: { cacheMode?: string, meta?: any }): (Promise<any>);
  getProfile(): Promise<Profile>;
  insert(obj: any, cacheMode?: string): Promise<any>;
  log(logEntry?: any, requestInformation?: any, deviceInformation?: any): Promise<any>;
  register(account?: string, password?: string, device?: Device): Promise<User>;
  requestPasswordReset(account: string): Promise<any>;
  resetPassword(token: string, password: string, device?: Device): Promise<any>;
  signIn(username: string, password: string): Promise<any>;
  signInGuest(): Promise<any>;
  signOut(): Promise<any>;
  update(obj: any, options?: { cacheMode?: string, meta?: any }): Promise<any>;
  xAuth(json: any): Promise<any>;
}
export const clinical6: Clinical6;
export default clinical6;

export class Client {
  constructor(enforcer);
  static instance: Client;
  _activeRequests: any[];

  // getters / setters
  apiBaseUrl: string;
  authToken: string;
  onError: Function;
  config: any;
  device: Device;
  mobileApplicationKey: string;
  settings: any;
  storageUtility: StorageUtility;
  user: User;

  abort(filter?: Function);
  fetch(url: string, method?: string, data?: any, headers?: any): Promise<any>;
  http(json: any): Promise<any>;
  request(cacheKey: string, fetchPromise: Promise<any>, options: any): Promise<any>;
  reset();
}

/** Helpers */

/**
 * Base Helper Class
 */
export class Helper {
  // private
  _meta: any;
  _relationships: any;

  // setters / getters
  meta: any;
  type: string;
  relationships: any;

  // methods
  clear(): Promise<any>;
  deserializeRelationshipStubs(json);
  getRelationship(obj): Promise<any>;
  initializer(): void;
  load(): Promise<any>;
  store(): Promise<any>;
  syncRelationships(json): Promise<any>;
  toJSON(): any;
}

export class Advertisement {
  constructor(response?);
  action: number;
  action_object: any;
  content_id: number;
  description: string;
  id: number;
  image: any;
  tags: Array<any>;
  title: string;
}

export class Agreement {
  constructor(response?);
  id: number;
  commonId: string;
  status: string;
  completedAt: string;
  documentUrl: string;
  createdAt: string;
  updatedAt: string;

  //old attributes for backwards support
  templateId: number;
  template: AgreementTemplate;
  signatures: AgreementSignature[];

  // setters / getters
  type: string;
  agreementTemplate: AgreementTemplate;
  mobileUser: User;
  agreementSignatures: AgreementSignature[];

}

export class AgreementSignature {
  id: number;
  role: string;
  status: string;
  signUrl: string;
  signedAt: string;
  signedByEmail: string;
  //old attributes for backwards support
  agreementId: number;
  email: string;

}

export class AgreementTemplate extends Helper {
  constructor(response?);
  id: number;
  templateName: string;
  description: string;
  message: string;
  redirectUrl: string;
  createdAt: string;
  updatedAt: string;
  permanentLink: string;
  reminderFrequency: string;
  archivedAt: string;
  expiration: number;
  approvalRequired: boolean;

  // setters / getters
  type: string;
  approvers: Object;
  consentFormVersions: ConsentFormVersion[];
  relationships: any;

  // methods
  save(): Promise<AgreementTemplate>;
}

export class PaperTemplate extends Helper {
  constructor(response?);
  id: number;
  templateName: string;
  description: string;
  documentUrl: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  consentFormVersions: ConsentFormVersion[];
  relationships: any;

  // methods
  save(): Promise<PaperTemplate>;
}

export class AgreementTemplateField extends Helper {
  constructor(response?);
  id: number;
  label: string;
  fieldName: string;
  required: boolean;
  defaultValue: string;
  attributeName: string;
  signerIndex: number;
  sourceType: string;
  reminderFrequency: string;
  archivedAt: string;
  expiration: number;
  approvalRequired: boolean;

  // setters / getters
  type: string;
  approvers: Object;
  relationships: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<AgreementTemplateField>;
}

export class AllowedAction extends Helper{
  constructor(response?);
  id: number;
  name: string;
  scopeName: string;

  // setters / getters
  type: string;
  permission: Permission;
  relationships: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<AllowedAction>;
}

export class AnalyticsMetric extends Helper {
  constructor(response?);
  id: string;
  type: string;
  value: number;
  unit: string;
  name: string;
}

export class AppMenu extends Helper{
  constructor(response?);
  id: number;
  title: string;
  action: string;
  position: number;
  image: ImageModel;
  enabled: boolean;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  actionDetail: any;
  badge: string;
  children: AppMenu[];
  parent: AppMenu;
  relationships: any;
  updateBadge: any;

  // methods
  delete(): Promise<any>;
  processDefaults();
  save(): Promise<AppMenu>;
}

export class ApproverAssignment extends Helper{
  constructor(response?);
  id: number;

  // setters / getters
  approver: ConsentApprover;
  approverGroup: ApproverGroup;
  type: string;
  relationships: any;

  // methods
  save(): Promise<ApproverAssignment>;
}

export class ApproverGroup extends Helper{
  constructor(response?);
  id: number;
  name: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  formVersions: ConsentFormVersion[];
  approverAssignments: ApproverAssignment[];
  approvers: ConsentApprover[];
  type: string;
  relationships: any;

  // methods
  addApprover(approver: ConsentApprover): Promise<ApproverAssignment>;
  addFormVersion(formVersion: ConsentFormVersion): Promise<ApproverGroupAssignment>;
  delete(): Promise<any>;
  getApproverAssignments(params?: object, options?: object): Promise<ApproverAssignment[] | ApproverAssignment>;
  getApprovers(): Promise<ConsentApprover[]>;
  removeApprover(obj: ApproverAssignment|ConsentApprover): Promise<any>;
  save(): Promise<ApproverGroup>;
}

export class ApproverGroupAssignment extends Helper{
  constructor(response?);
  id: number;

  // setters / getters
  formVersion: ConsentFormVersion;
  approverGroup: ApproverGroup;
  type: string;
  relationships: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<ApproverGroupAssignment>;
}

export class AvailableStrategy extends Helper {
  constructor(response?);
  id: number;

  // setters / getters
  type: string;
  consentForms: ConsentForm[];
  consentStrategy: ConsentStrategy;
}

export class AwardedBadge extends Helper {
  constructor(response?);
  id: number;
  type: string;
  count: number;
  createdAt: string;
  earnedOnDate: string;
  updatedAt: string;

  // setters / getters
  awardee: User;
  badge: Badge;
  relationships: any;

  delete(): Promise<any>;
  save(): Promise<AwardedBadge>;
}

export class Badge extends Helper {
  constructor(response?);
  id: number;
  type: string;
  title: string;
  basedOn: string;
  cacheToken: any;
  createdAt: string;
  description: string;
  disabledImage: ImageModel;
  enabled: boolean;
  featured: any;
  image: ImageModel;
  position: number;
  redemptionPoints: string;
  startAt: string;
  startPoint: string;
  threshold: number;
  timeUnit: string;
  updatedAt: string;

  delete(): Promise<any>;
  save(): Promise<Badge>;
}

export class BulkImport extends Helper{
  constructor(response?);
  id: number;
  platformModel: string;
  separator: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  relationships: any;
  genericFile: GenericFile;
  status: JobStatus;

  // methods
  save(): Promise<any>;
}

export class Cohort extends Helper{
  constructor(response?);
  id: number;
  name: string;
  cohortType: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  assignments: CohortAssignment[];
  users: User[];
  type: string;

  // methods
  addUser(user: User): Promise<CohortAssignment>;
  getUsers(): Promise<User[]>;
  getAssignments(params?: object, options?: object): Promise<CohortAssignment[] | CohortAssignment>;
  removeUser(obj: CohortAssignment|User): Promise<any>;
  save(): Promise<Cohort>;
}

export class CohortAssignment extends Helper{
  constructor(response?);
  id: number;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  cohort: Cohort;
  user: User;

  // methods
  delete(): Promise<any>;
  save(): Promise<CohortAssignment>;
}

export class ConsentApprover extends Helper {
  constructor(response?);
  id: number;
  firstName: string;
  lastName: string;
  email: string;
  title: string;
  password: string;

  // setters / getters
  type: string;
  relationships: any;
  approverGroups: ApproverGroup[];
  consentFormVersion: ConsentFormVersion;

  // methods
  save(): Promise<ConsentApprover>;
}

export class ConsentForm extends Helper{
  constructor(response?);
  id: number;
  enabled: boolean;
  name: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  relationships: any;

  // additionalSignatures: ConsentAdditionalSigner[];
  consentFormVersions: ConsentFormVersion[];
  consentStrategy: ConsentStrategy;
  strategy: ConsentStrategy;

  // methods
  getVersions(options?: object): Promise<ConsentFormVersion[]>;
  save(): Promise<ConsentForm>;
}

export class ConsentFormVersion extends Helper{
  constructor(response?);
  id: number;
  archivedAt: string;

  // setters / getters
  type: string;
  relationships: any;

  approvers: Object[];
  approverGroups: ApproverGroup[];
  consentForm: ConsentForm;
  language: Language;
  templatable: AgreementTemplate | PaperTemplate;
  site: Site;
  getApproverGroupAssignments: ApproverGroupAssignment[];

  // methods
  save(): Promise<ConsentFormVersion>;
}

export class ConsentGrant extends Helper{
  constructor(response?);
  id: number;
  signingPassword: string;
  signUrl: string;
  createdAt: string;
  updatedAt: string;
  consentedAt: string;
  progress: string;
  documentUrl: string;
  // additionalSignatures: Array<{ signing_password: number }> // for preparing passwords only

  // setters / getters
  type: string;
  relationships: any;

  agreement: Agreement;
  grantedBy: User;
  grantedFor: User;
  form: ConsentForm;
  formVersion: ConsentFormVersion;
  strategy: ConsentStrategy;

  // methods
  delete(): Promise<any>;
  save(): Promise<ConsentGrant>;
}

export class ConsentAdditionalSigner extends Helper {
  constructor(response?);
  id: number;
  name: string;
  createdAt: string;
  updatedAt: string;
  deletedAt: string;

  // setters / getters
  type: string;
  relationships: any;
  form: ConsentForm;

  // methods
  save(): Promise<ConsentAdditionalSigner>;
}

export class ConsentStrategy extends Helper{
  constructor(response?);
  id: number;
  name: string;
  strategyType: string;
  location: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  relationships: any;

  cohort: Cohort;
  forms: ConsentForm[];

  // methods
  delete(): Promise<any>;
  save(): Promise<ConsentStrategy>;
}

export class Comment {
  constructor(response?);
  id: number;
  body: string;
  createdAt: string;

  // setters / getters
  author: User;
  relationships: any;
  thread: Thread;
  type: string;
}

export class Commentable {
  addComment(text: string, author?: any, cacheMode?: string): Promise<Thread>;
  getComments(cacheMode?): Promise<Thread[]>;
  reply(text: string, thread: Thread, author?: any, cacheMode?: string): Promise<Comment>;
}

export class Consultation extends Helper {
  constructor(response?);
  id: number;
  name: string;
  sessionIdentity: string;
  scheduledAt: string;
  status: string;
  deletedAt: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  static type: string;
  relationships: any;
  participants: ConsultationParticipant[];

  // methods
  delete(): Promise<any>;
  join(participant?: any, meta?: any): Promise<Consultation>;
  save(): Promise<Consultation>;
}

export class ConsultationParticipant extends Helper {
  constructor(response?);
  id: number;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  static type: string;
  relationships: any;
  consultation: Consultation;
  participant: User;

  // methods
  delete(): Promise<any>;
  save(): Promise<Consultation>;
}

export interface ContentResource {
  created_at: string;
  id: string;
  name: string;
  path: {
    url: string;
  };
  position: string;
  resourceable_id: string;
  resourceable_type: string;
  updated_at: string;
}

export class Content extends Helper implements Commentable {
  constructor(contentType, response?, options?, meta?);
  [id: string]: any;
  id: number;
  createdAt: string;
  updatedAt: string;
  description: string;
  imageUrl: string;
  position: number;
  showInAdminApp: boolean;
  showInMobileApp: boolean;
  title: string;
  visibilityStatus: string; // 'enabled'|'disabled'?

  // type: Article
  article?: ContentResource[];
  articleName?: string;
  fileType?: string;
  publishedDate?: string; // YYYY-MM-DD

  // type: ~FlowStep
  footer?: string;
  helpText?: string;
  progressBar?: boolean;
  inputLocation?: 'right' | 'bottom';
  sliderOrientation?: 'vertical' | 'horizontal';
  call?: string;

  // type: Consent
  subtitle?: string;

  // type: FAQ
  answer?: string;
  question?: string;

  // type: HTML
  html?: string;

  // type: Glossary
  definition?: string;
  term?: string;

  // type: Learn More
  RtfDescription?: string;

  // type: Policy
  policy?: string;

  // type: Video
  length?: number;
  video?: ContentResource[];

  // setters / getters
  contentType: ContentType;
  _data: any;
  _meta: any;
  _options: any;
  objectType: string;
  options: any;
  owner: string;
  ownerType: string;
  resource: string;
  status: string;
  type: string;
  relationships: any;

  set(key, value);
  get(key): any;
  delete(): Promise<any>;
  getStatus(): Promise<any>;
  save(): Promise<Content>;

  // Commentable
  addComment(text: string, author?: any, cacheMode?: string): Promise<Thread>;
  getComments(cacheMode?): Promise<Thread[]>;
  reply(text: string, thread: Thread, author?: any, cacheMode?: string): Promise<Comment>;
}

export class ContentAttribute extends Helper {
  constructor(response?);
  id: number;
  name: string;
  attributeType: string;
  required: boolean;
  position: number;
  displayName: string;
  defaultValue: string;
  showOnIndex: boolean;
  status: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  relationships: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<ContentAttribute>;
}

export class ContentType extends Helper {
  constructor(response?);
  id: number;
  name: string;
  permanentLink: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  relationships: any;

  // methods
  getAttributes(params?: any, options?: any): Promise<ContentAttribute[] | ContentAttribute>
}

export class CsvExampleFile extends Helper{
  constructor(response?);
  id: number;
  humanReadableName: string;
  platformModel: string;
  fileUrl: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  relationships: any;
}

export class CustomClass extends Helper {
  constructor(response?);
  id: number;
  [id: string]: any;

  // setters / getters
  type: string;
  relationships: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<any>;
}

export class Device extends Helper {
  constructor(response?);
  id: number;
  udid: string;
  technology: string;
  accessToken: string;
  pushId: string;
  createdAt: string;
  updatedAt: string;
  appVersion: string;

  // setters / getters
  type: string;
  uuid: string;

  delete(): Promise<any>;
  save(): Promise<Device>;
  select(): Device;
}

export class Entry extends Helper {
  constructor(response?);
  id: number;
  date: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  flowDataGroup: FlowDataGroup;
  entryGroup: EntryGroup;
  flow: Flow;
  template: EntryTemplate;
  owner: User;
  relationships: any;

  add(): Promise<Entry>;
}

export class EntryGroup extends Helper {
  constructor(response?);
  id: number;
  category: string;
  createdAt: string;
  name: string;
  permanentLink: string;
  position: number;
  updatedAt: string;

  // setters / getters
  type: string;
  children: EntryGroup[];
  parent: EntryGroup;
  entries: Entry[];
  templates: EntryTemplate[];
  relationships: any;

  getTemplates(): Promise<EntryTemplate[] | EntryTemplate>
}

export class EntryTemplate extends Helper {
  constructor(response?);
  id: number;
  category: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  entryGroup: EntryGroup;
  flow: Flow;
  relationships: any;

  addEntry(user: User, attributes: any): Promise<Entry[] | Entry>;
}

export class Event {
  constructor(response?);
  id: number;
  date: string;
  extras: any;
  status: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  schedulers: Schedule[];
  relationships: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<Event>;
  mark(ruleId: string, eventAction: string): Promise<Object>;
}

export class Flow extends Helper implements Commentable {
  constructor(response, settings?);
  _fields: Object;
  _options: Object;
  _status: string;
  _toSave: Array<any>;
  id: number;
  steps: Array<FlowStep>;
  entry: Entry;
  existingId: string;
  fields: any;
  first_step: number;
  first: FlowStep;
  name: string;
  objectType: string;
  options: any;
  owner: string | number;
  ownerType: string;
  permanentLink: string;
  settings: any;
  status: any;
  total: number;
  type: string;
  draft: object;
  published: object;
  publishedAt: string;
  flowDataGroupId: number; // for collecting values on same data group

  addFieldsToSave(array: Array<Number>): void;
  clone(): Flow;
  complete(): Promise<any>;
  connectGraph();
  end(flowStep: FlowStep): void;
  findInputs(whereFn: Function): Array<FlowInput>;
  get(key: string|number): any;
  getBFS(flowStep: FlowStep): { number: number};
  getData(): Promise<any>;
  getLongestDistance(flowStep: FlowStep): number;
  getJohnsonMatrix(): any;
  getStatus(): Promise<any>;
  loadSteps(): Promise<Array<FlowStep>>;
  onSave(response: any): any;
  postInsights(action: string): Promise<any>;
  reset(): void;
  save(): Promise<any>;
  saveFlowAttributes(): Promise<Flow>;
  set(key: any, value: any);
  start(): Promise<any>;
  toPlantUML(): string;
  transition(action: string): Promise<any>;

  // Commentable
  addComment(text: string, author?: any, cacheMode?: string): Promise<Thread>;
  getComments(cacheMode?): Promise<Thread[]>;
  reply(text: string, thread: Thread, author?: any, cacheMode?: string): Promise<Comment>;
}

export class FlowContainer extends Helper{
  constructor(response?);
  id: number;
  name: string;
  permanent_link: string;
}

export class FlowData {
  constructor(response);
  id: number;
  createdAt: string;
  ownerType: string;
  submittedAt: string;
  updatedAt: string;
  value: string;
}

export class FlowDataGroup extends Helper {
  constructor(response);
  id: number;
  createdAt: string;
  ownerType: string;
  submittedAt: string;
  updatedAt: string;
  finalSubmission: boolean

  type: string;
  capturedValues: FlowData[];
  flowData: FlowData[];
  creator: User;
  owner: User;
  flow: Flow;
  relationships: Object;
  meta: Object;
  status: Status;
  isComplete: boolean;
}

export class FlowInput {
  constructor(response?);
  attribute: string;
  choice_list: Array<any>;
  code: string;
  id: number;
  instructions: string;
  body: string;
  locked: any;
  maxLabel: string;
  max: number;
  minLabel: string;
  min: number;
  question_type: string;
  required: string;
  storage_attribute: string;
  style: string;
  title: string;
  validation_details: any;
  validation_expression: string;
  findChoiceByCode(value: string): any;
  getData(): Promise<any>;
}

export class FlowKeyValue extends Helper {
  constructor(response?);

  // Model
  id: number;
  [id: string]: any;

  // setters / getters
  flow: Flow;
  flowDataGroup: FlowDataGroup;
  owner: any;
  type: string;
  relationships: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<FlowKeyValue>;
}

export class FlowStep extends Helper implements Commentable {
  constructor(response?);

  // Model
  id: number;
  title: string;
  description: string;
  contentType: any;
  image: any;
  paths: Array<{button_name: string, capture?: boolean, last?: boolean, steps: any[]}>;
  sendOnCapture: boolean;
  flowProcessId: number;
  helpText?: string;
  footer?: string;
  progressBar?: boolean;
  inputLocation?: 'bottom' | 'right'
  sliderOrientation?: 'vertical' | 'horizontal'
  call?: string;

  // Helper
  flow: Flow;
  inputs: Array<FlowInput>;
  _options: Object;
  type: string;

  // getters / setters
  existingId: number;
  fields: any;

  addFieldsToSave(): void;
  get(key: string|number): any;
  getNextStep(steps: Array<any>): FlowStep;
  getPath(name: string): any;
  go(name: string): Promise<FlowStep>;
  hasFields(): boolean;
  hasSteps(path: any): boolean;
  initFields(): void;
  isAutoSave(path: any): boolean;
  isFieldRequired(inputId: string): boolean;
  onSave(response: any): any;
  postInsights(action: string): Promise<any>;
  save(): Promise<any>;
  set(key: any, value: any);
  validateRequiredFields(path: any): void;

  // Commentable
  addComment(text: string, author?: any, cacheMode?: string): Promise<Thread>;
  getComments(cacheMode?): Promise<Thread[]>;
  reply(text: string, thread: Thread, author?: any, cacheMode?: string): Promise<Comment>;
}

export class GenericFile extends Helper{
  constructor(response?);
  id: number;
  name: string;
  public: boolean;
  email: string;
  visibleUntil: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  relationships: any;
}

export class ImageModel {
  constructor(response?);
  id: number;
  url: string;
  fullscreen: string;
  fullscreenHD: string;
  main: string;
  mainHD: string;
  small: string;
  smallHD: string;
}

export class Insight {
  constructor(response?);
  tracker: any;
}

export class Language extends Helper {
  constructor(response?: any, getTranslations?: boolean);
  _translations: any;
  id: number;
  iso: string;
  name: string;
  isDefault: boolean;

  // getter / setter
  type: string;

  // methods
  getTranslations(): Promise<any>;
  translate(key: string): string;
}

export class Invitation extends Helper {
  constructor(response?: any);
  [id: string]: any;
  id: number;
  countryCode?: string; // mobile_user only
  dynamicProfile?: any; // mobile_user only
  email: String;
  firstName: String;
  lastName: String;
  memberType?: string; // mobile_user only
  gender?: string; // mobile_user only
  phone?: string; // mobile_user only
  suffix?: String; // deprecated?
  dob?: String; // mobile_user only
  accountName?: String; // deprecated?
  relationship?: String; // deprecated?
  studyDefinitionVersionId?: number; // mobile_user only
  timezone?: string; // mobile_user only

  // getter / setter
  followed: User; // deprecated?
  language: Language;
  relationships: any;
  role: Role;
  type: string;
  cohort: Cohort; // mobile_user only
  strategy: ConsentStrategy; // mobile_user only
  site: Site;

  // methods
}

export declare namespace Invitation {
  interface Attributes {
    [id: string]: any;
    id: number;
    countryCode?: string; // mobile_user only
    dynamicProfile?: any; // mobile_user only
    email: String;
    firstName: String;
    lastName: String;
    memberType?: string; // mobile_user only
    gender?: string; // mobile_user only
    phone?: string; // mobile_user only
    suffix?: String; // deprecated?
    dob?: String; // mobile_user only
    accountName?: String; // deprecated?
    relationship?: String; // deprecated?
    studyDefinitionVersionId?: number; // mobile_user only
    timezone?: string; // mobile_user only
  }
  interface Relationships {
    [id: string]: any;
    cohort?: Cohort; // mobile_user only
    language?: Language;
    site?: Site;
    strategy?: ConsentStrategy; // mobile_user only
    userRole: Role;
  }
}

export class JobStatus extends Helper{
  constructor(response?);
  id: number;
  status: string;
  startedAt: string;
  completedAt: string;
  errorContent: string;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  type: string;
  relationships: any;
}

export class Location extends Helper{
  constructor(response?);
  [id: string]: any;
  country: string;
  state: string;
  city: string;
  street: string;
  latitude: number;
  longitude: number;
  title: string;
  zipCode: string;
  addressLine1: string;
  addressLine2: string;
  addressLine3: string;

  // getters / setters
  type: string;
  localizable: object;
  relationships: any;

  // methods
  save(): Promise<Location>;
}

export class Message {
  constructor(response?);
  id: number;
  title: string;
  body: any;
  created_at: string;
  updated_at: string;
  subject: string;
  recipients: Array<any>;
  conversation_id: string;
  attachment: any;
}

export class Notification {
  constructor(response?);
  callback: any;
  data: any;
  id: number;
  image: any;
  message: string;
  status: string;
  title: string;
  createdAt: string;
  updatedAt: string;
  archivedAt: string;
  readAt: string;
  archive: boolean;
  read: boolean;
  save(): Promise<Notification>;
  delete(): Promise<any>;
}

export class PlatformMenu extends Helper {
  constructor(response?);
  id: number;
  title: string;
  position: number;
  url: string;
  enabled: boolean;
  icon: string;
  accessClass: string;

  // setters / getters
  type: string;
  parent: PlatformMenu;
  children: PlatformMenu[];
  authorizable: Flow | Section | Content;
  relationships: any;

  delete(): Promise<any>;
  save(): Promise<PlatformMenu>;
}

export class Profile extends Helper {
  constructor(response?);
  [id: string]: any;
  age: number;
  avatar: any;
  /** @deprecated */
  city: string;
  countryCode: string;
  createdAt: string;
  dob: string;
  ethnicity: string;
  firstName: string;
  fitbitLastSynced: string;
  gender: string;
  hoursTimeTraveled: number | null;
  lastName: string;
  middleInitial: string;
  miscellaneous: string;
  occupation: string;
  phone: string;
  prefix: string;
  /** @deprecated */
  state: string;
  /** @deprecated */
  street: string;
  suffix: string;
  /** @default 'UTC' */
  timezone: string;
  updatedAt: string;
  username: string;
  /** @deprecated */
  zipCode: string;

  // getters / setters
  type: string;
  language: Language;
  profileable: User;
  user: User;

  // methods
  // addComment(text: string): Promise<Comment>;
  // getComments(): Promise<Comment[]>;
  getFullName(): string;
  /** @deprecated */
  getFullAddress(): string;
  save(): Promise<any>;
}

export class ProjectContent {
  constructor(type?: string, options?: any, data?: any, meta?: any);
  _data: any;
  options: any;
  meta: any;
  addChild(type: string, child: any): Promise<any>;
  getAll(): Promise<Array<ProjectContent>>;
  getChild(type: string, id: number): Promise<any>;
  getChildren(type: string): Promise<any>;
  getData(): Promise<any>;
  getSibling(id: number): Promise<ProjectContent>;
  removeChild(type: string, id: number): Promise<any>;
  save(): Promise<any>;
  updateChild(type: string, child: any, id?: number): Promise<any>;
}

export class Query {
  constructor(resource: string, orderBy?: Array<any>, page?: number, perPage?: number, minimal?: boolean, brandRelative?: boolean);
  filter: any;
  filters: Array<any>;
  resource: any;
  orderBy: any;
  page: any;
  perPage: any;
  minimal: any;
  brandRelative: any;
  whole_word: any;
  case_sensitive: any;
  reference_module: any;
  attribute: any;
  searchMethod: string;
  searchValues: Array<any>;
  setResource(content: any): Query;
  validate(): void;
  newFilter(attribute?: string): Query;
  find(): Promise<Content[]>;

  addSearchValue(searchValue: string): Query;
  removeSearchValue(searchValue: string): Query;
  clearSearchValues(): Query;
  searchMode(): Query;
  startsWithMode(): Query;
  endsWithMode(): Query;
  wholeWordMode(): Query;
  wholeWordSearch(): Query;
  makeCaseSensitive(): Query;
  lessThanOrEqualTo(): Query;
  greaterThanOrEqualTo(): Query;
  setValue(num: number): Query;
  ascending(col: string): Query;
  descending(col: string): Query;
  limit(num: number): Query;
  showMinimal(): Query;
  showBrandRelative(): Query;
  referenceModule(val: string): Query;
  searchByAddressMode(): Query;
  searchByDateMode(): Query;
  searchByLocationMode(): Query;
  searchByNumberMode(): Query;
  searchByTagIDMode(): Query;
  searchByTagNamesMode(): Query;
  setRadius(radius: any): Query;
  setExactLatitude(latitude: any): Query;
  setExactLongitude(longitude: any): Query;
  setNeLat(latitude: any): Query;
  setNeLong(longitude: any): Query;
  setNwLong(longitude: any): Query;
  setSwLat(latitude: any): Query;
  setSwLong(longitude: any): Query;
}

export class Permission extends Helper{
  constructor(response?);
  id: number;
  enabled: boolean;
  scopeName: string;

  // setters / getters
  type: string;
  relationships: any;
  allowedActions: AllowedAction[];
  authorizable: Flow | Section | Content;
  role: Role

  delete(): Promise<any>;
}

export class PredefinedScreen extends Helper{
  constructor(response?);
  id: number;
  name: string;
  permanent_link: string;
}

export class RelatedUser extends Helper {
  constructor(response?);
  id: number;
  createdAt: boolean;
  updatedtAt: boolean;
  relationship: string;

  // setters / getters
  type: string;
  relationships: any;

  // methods
  save(): Promise<RelatedUser>;
  delete(): Promise<any>;
}

export class Role extends Helper {
  constructor(response?);
  id: number;
  permanentLink: string;
  name: string;
  isSuper: boolean;
  isAdmin: boolean;
  isMobile: boolean;
  isDefault: boolean;
  description: string;
  canViewPii: boolean;

  // setters / getters
  type: string;
  relationships: any;

  // methods
  getInvitableRoles(): Promise<Role>;
  getPermissions(): Promise<Permission>;
  save(): Promise<Role>;
}

export class Rule extends Helper {
  constructor(response?);
  end_date: string;
  days_of_week: string;
  time: string;
  id: number;
  start_date: string;
  frequency: string;
  timezone: string;
  permanent_link: string;
  relative_unit: string;
  relative_schedule: string;

  // setters / getters
  type: string;
  schedulers: Schedule[];
  relationships: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<Rule>;
  nextEvent(): Promise<Event>;
  recentEvents(timeUnit?: string): Promise<any>;
  upcomingEvents(timeUnit?: string): Promise<any>;
  eventsByDate(date?: string): Promise<any>;
}

export class Schedule extends Helper {
  constructor(response?);
  id: number;
  activeTimeStart: string;
  activeTimeEnd: string;
  createdAt: string;
  daysOfWeek: string;
  enabled: boolean;
  endDate: string;
  exceptionDays: string;
  relativeSchedule: string;
  relativeUnit: string;
  skipExceptions: boolean;
  startDate: string;
  startDateOffset: Number;
  startDateOffsetUnit: string;
  time: string;
  timezone: string;
  updatedAt: string;  nextEvent(): Promise<any>;

  // setters / getters
  type: string;
  mobileUser: User;
  scheduleable: Rule;
  relationships: any;

  recentEvents(timeUnit?: string): Promise<any>;
  upcomingEvents(timeUnit?: string): Promise<any>;
  eventsByDate(date?: string): Promise<any>;
}

export class Section {
  constructor(response?);
  accessClass: string;
  object: string;
  label: string;
  type: string;
  status: string;
  owner: string;
  ownerType: string;
  sub_statuses: any;
  sections: any;
  getStatus(): Promise<any>;
  getChildren(): Promise<any>;
  getData(): Promise<any>;
  connectChildren(children: Array<any>): void;
  transition(action: string): Promise<any>;
}

export class Session extends Helper {
  constructor(response?);
  id: number;
  signInCount: string;
  lastSignInAt: string;
  failedAttempts: string;
  lockedAt: string;
  username: string;
  email: string;
  authenticationToken: string;
  confirmedAt: string;
  invitationSentAt: string;
  invitationAcceptedAt: string;
  invitationDueAt: string;
  passwordChangedAt: string;
  disabledAt: string;

  // setters / getters
  type: string;
  user: User;
  relationships: any;

  // methods
}

export class Setting extends Helper {
  constructor(response?);
  id: string;
  value: string;

  // setters / getters
  type: string;
  relationships: any;

  // methods
  save(): Promise<Setting>;
}

export class SignAccount extends Helper{
  constructor(response?);
  id: number;
  email: string;
  createdAt: string;
  updatedAt: string;
  linked: boolean;

  // setters / getters
  type: string;

  // methods
  delete(): Promise<any>;
  save(): Promise<SignAccount>;
}

export class Site extends Helper {
  constructor(response?);
  id: number;
  createdAt: string;
  updatedAt: string;
  siteId: string;
  externalIdentifier: string;
  randomizationIdentifier: string;
  name: string;
  email: string;
  phoneNumber: string;
  faxNumber: string;
  timezone: string;
  contentType: string;

  // setters / getters
  type: string;
  agreementTemplates: AgreementTemplate[];
  contacts: SiteContact[];
  location: Location;
  relationships: any;
  siteLanguages: SiteLanguage[]; // deprecated?
  protocolVersions: CustomClass[];

  // methods
  delete(): Promise<any>;
  getMembers(params?: any, options?: any): Promise<SiteMember[] | SiteMember>
  getSiteLanguages(params?: any, options?: any): Promise<SiteLanguage[]>
  save(): Promise<Site>;
}

export class SiteContact extends Helper {
  constructor(response?);
  id: number;
  firstName: string;
  lastName: string;
  contentType: string;
  contactType: string;
  email: string;
  phone: string;
  fax: string;
  primaryContact: boolean;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  site: Site;
  relationships: any;

  // methods
  save(): Promise<SiteContact>;
}

export class SiteLanguage extends Helper {
  constructor(response?);
  id: number;
  createdAt: string;
  updatedAt: string;

  // setters / getters
  language: Language;
  site: Site;
  relationships: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<SiteLanguage>;
}

export class SsoOption extends Helper{
  constructor(response?);
  id: number;
  name: string;
  userType: string;
  permanentLink: string;

  // setters / getters
  type: string;

  // methods
}

export class SiteMember extends Helper {
  constructor(response?);
  id: number;
  accountStatus: string;
  consentStatus: string;
  createdAt: string;
  firstConsentedAt: string;
  lastConsentedAt: string;
  memberType: string;
  preScreenEligible: boolean | null;
  screenEligible: boolean | null;
  updatedAt: string;

  // setters / getters
  activeStudyDefinitionVersion: CustomClass // TODO: implement CustomClass type: 'study__definition_versions'
  studyDefinitionVersion: CustomClass // this one is for saving only
  type: string;
  mobileUser: User;
  relationships: any;
  site: Site;
  consentStrategy: ConsentStrategy; // TODO: clarify difference with mobile_user.strategies

  // untested
  accessible: any;
  brand: any;
  callback: any;
  pointsRecords: any;
  rewardEarnings: any;
  segmentations: any;
  schedules: any;
  tag: any;
  taggings: any;

  // methods
  delete(): Promise<any>;
  save(): Promise<SiteMember>;
}

export class Status extends Helper{
  constructor(response?);
  id: number;
  value: string;

  // setters / getters
  owner: any;
  statusable: any;
  type: string;
}

export class Storage {
  readonly contentTypes: Promise<ContentType[]>;
  readonly devices: Promise<Device[]>;
  readonly menus: Promise<AppMenu[]>;
  readonly notifications: Promise<Notification[]>;
}

export class TemporaryId extends Helper{
  constructor(response?);
  id: number;
  token: string;
  expiresAt: string;

  // setters / getters
  type: string;
  relationships: any;
  user: User;

  // methods
  save(): Promise<TemporaryId>;
}

export class Thread extends Helper implements Commentable {
  constructor(response?);
  id: number;
  status: string;
  createdAt: string;
  resolvedAt: string;

  // setters / getters
  commentable: any;
  comments: Comment[];
  owner: User;
  relationships: any;
  type: string;

  // Commentable
  addComment(text: string, author?: any, cacheMode?: string): Promise<Thread>;
  getComments(cacheMode?): Promise<Thread[]>;
  reply(text: string, thread: Thread, author?: any, cacheMode?: string): Promise<Comment>;
}

export class Timezone extends Helper {
  constructor(response?);
  id: number;
  type: string;
  name: string;
  offset: number;
  region: string;
}

export class User extends Helper{
  constructor(response?);
  id: number;
  accountName?: string; // mobile_users only
  accountStatus?: string; // users only / mobile_users have it within site_members
  antiSpamAcceptedAt: string;
  createdAt: string;
  /** @deprecated */
  disabled: boolean;
  disabledAt: string;
  /** @deprecated */
  enabled: boolean;
  email: string;
  /** @deprecated */
  encryptionKey: string;
  /** @deprecated */
  guest: string;
  invitationAcceptedAt: string;
  invitationDueAt: string;
  invitationSentAt: string;
  passwordExpiredAt?: string; // mobile_users only
  privacyPolicyAcceptedAt: string
  termsOfUseAcceptedAt: string;
  updatedAt: string;
  /** @deprecated */
  username: string;
  uuid: string;
  /** @deprecated */
  verifiedAt: string;
  /** @deprecated */
  withdraw: boolean;
  withdrawnAt?: string; // mobile_users only
  initialAccountName?: string; // mobile_users only
  accountUrls: any;

  // setters / getters
  cohortAssignments?: CohortAssignment[];  // mobile_users only
  cohorts?: Cohort[];  // mobile_users only
  consentSigner?: CustomClass;  // mobile_users only // TODO: implement CustomClass type: 'consent__participant_signers'
  devices: Device[];
  grantGroups: CustomClass[]; // TODO: implement CustomClass type: 'consent__grant_groups'
  homeLocation: Location;
  language?: Language; // mobile_users only (reflects the one from profile)
  profile: Profile;
  relationships: any;
  role: Role;
  sites?: Site[]; // users only
  siteMember?: SiteMember; // mobile_users only
  status?: Status; // mobile_users only
  strategies?: ConsentStrategy[]; // mobile_users only
  threads?: Thread[]; // mobile_users only
  type: string;


  // methods
  getAvailableStrategies(options?: object): Promise<AvailableStrategy[]>;
  getCohortAssignments(params?: object, options?: object): Promise<CohortAssignment[] | CohortAssignment>;
  getConsentGrants(options?: object): Promise<ConsentGrant[]>;
  getConsentGrants()
  addBadge(badge: Badge, cacheMode?: string): Promise<AwardedBadge[] | AwardedBadge>;
  addEntry(attributes: any, template: EntryTemplate): Promise<Entry[] | Entry>;
  addSchedule(schedule: any): Promise<Schedule>;
  delete(): Promise<any>;
  disable(): Promise<any>;
  enable(): Promise<any>;
  getAwardedBadges(awardedBadge?: any, cacheMode?: string): Promise<AwardedBadge[] | AwardedBadge>;
  getDataGroup(user?: any, params?: any, cacheMode?: string): Promise<FlowDataGroup>;
  getEntries(filter?: any, cacheMode?: string): Promise<Entry[] | Entry>;
  getNotifications(): Promise<Notification[] | Notification>;
  getProfile(): Promise<Profile>;
  getRegistrationStatus(): Promise<string>;
  getSchedules(user?: any, cacheMode?: string): Promise<Schedule[]|Schedule>;
  invite(attributes?: any, relationships?: any): Promise<User>;
  isPasswordSet(): boolean;
  removeAwardedBadge(badge: Badge, cacheMode?: string): Promise<any>;
  requestPasswordReset(): Promise<any>;
  resendInvite(options?: any): Promise<any>;
  resetPassword(token: string, password: string, device?: any): Promise<any>;
  revokeConsent(cacheMode?: string): Promise<any>;
  save(): Promise<User>;
  saveProfile(): Promise<Profile>;
  sendConfirmation(): Promise<User>;
  toInvitation(attributes?: Invitation.Attributes, relationships?: Invitation.Relationships): Invitation;
  unlockUser(unlockToken: string): Promise<User>;
}

export class Version extends Helper {
  constructor(response?);
  id: number;
  category: string;
  createdAt: string;
  event: string;
  ip: string;
  originalObject: object;
  objectChanges: object;
  userAgent: string;
  reasonForChanges: object;

  // setters / getters
  user: User;
  item: Object;
  relationships: any;
}

export class VuforiaTarget {
  constructor(response?);
  vuforiaId: number;
  vuforiaTargetName: string;
  set(key: string, value: string): void;
  get(key: string): any;

}

/** Services */
export class AdvertisementService {
  getSettings(): Promise<any>;
  get(imageVersions?: string, tagNames?: Array<any>): Promise<any>;
}
export const advertisementService: AdvertisementService;

export class AgreementService {
  get(id: number, recipients: Array<any>, options?: any): Promise<Agreement>;
}
export const agreementService: AgreementService;

export class AnalyticsService {
  getCurrent(): Promise<any>;
  getMetrics(params?: { name?: string }, cacheMode?: string): Promise<AnalyticsMetric>;
	postInsights(
      action: string,
      section: string,
      withCurrentLocation?: boolean,
      value?: string,
      label?: string,
      triggeredAt?: string,
      brandId?: string,
      location?: any): Promise<any>;
}
export const analyticsService: AnalyticsService;

export class AppMenuService extends JsonApiService {
  clearStorageMenus(menu: AppMenu);
  delete(appMenu: AppMenu, cacheMode?: string): Promise<any>;
  get(params?: any, cacheMode?: string): (Promise<AppMenu[] | AppMenu>);
  getActionDetails(params?: any): Promise<any>;
  insert(appMenu?: AppMenu, cacheMode?: string): Promise<AppMenu>;
  update(appMenu: AppMenu, cacheMode?: string): Promise<AppMenu>;
  updateOrder(order: any, cacheMode?: string): Promise<AppMenu[]>;
}
export const appMenuService: AppMenuService;

export class CallbackService {
  execute(id: number): Promise<any>;
  get(id?: number): Promise<any>;
  getCallback(id: number): Promise<any>;
  getCallbacks(): Promise<any>;
}
export const callbackService: CallbackService;

export class ConsultationService extends JsonApiService {
  join(consultation: Consultation, participant: ConsultationParticipant, options?: any): Promise<Consultation>;
  complete(consultation: Consultation, participant: ConsultationParticipant, options?: any): Promise<Consultation>;
}
export const consultationService: ConsultationService;

export class ContentService extends JsonApiService {
  get(params: any, cacheMode?: string): (Promise<Content[] | Content>);
  getById(id: string|number): Promise<Content>;
  getContent(resource: string): Promise<Content[]>;
  getContentFavorites(resource: string, page?: number, perPage?: number): Promise<Content[]>;
  getTypes(params?: any, cacheMode?: string): Promise<ContentType[] | ContentType>;
  find(query: Query): Promise<Content[]>;
  filter(query: Query): Promise<Content[]>;
  getRandom(resource: string): Promise<Content>;
  getVuforiaDynamicContent(id: number): Promise<VuforiaTarget>;
  insert(content: any, cacheMode?: string): Promise<any>;
  update(content: any, cacheMode?: string): Promise<any>;
}
export const contentService: ContentService;

export class DeviceService extends JsonApiService {
  delete(device: Device, cacheMode?: string): Promise<any>;
  get(params?: any, cacheMode?: string): (Promise<Device[] | Device>);
  insert(mobileApplicationKey?: string, cacheMode?: string): Promise<Device>;
  update(device: Device, cacheMode?: string): Promise<Device>;
}
export const deviceService: DeviceService;

export class DiscussService {
  get(data: any, cacheMode?: string): Promise<Thread[]>;
  getComments(thread: Thread, cacheMode?: string): Promise<Comment[]>;
  insert(data: any, cacheMode?: string): Promise<Thread>;
  insertComment(text: string, thread: Thread, author?: any, cacheMode?: string): Promise<Comment>;
}
export const discussService: DiscussService;

export class EDiaryService extends JsonApiService {
  get(cacheMode?: string): (Promise<EntryGroup[] | EntryGroup>);
  getTemplates(params: any, cacheMode?: string): (Promise<EntryTemplate[] | EntryTemplate>);
}
export const eDiaryService: EDiaryService;

export class EventService extends JsonApiService {
  delete(event: Event, cacheMode?: string): Promise<any>;
  get(params?: any, cacheMode?: string): (Promise<Event[] | Event>);
  insert(event?: any, cacheMode?: string): Promise<Event>;
  update(event: Event, cacheMode?: string): Promise<Event>;
  trackEvents(resource: string, action: string, context?: any): Promise<any>;
}
export const eventService: EventService;

export class FlowService {
  collect(flow: Flow, options?: any): Promise<any>;
  collectFields(flow: Flow, fields: any, options?: any): Promise<any>;
  deleteKeyValue(keyValue: FlowKeyValue, cacheMode?: string): Promise<FlowKeyValue>;
  get(params?: any, cacheMode?: string): Promise<Flow|Flow[]>;
  getFlow(resource: string, page?: any, perPage?: any): Promise<Flow>;
  getDataGroup(params: any, cacheMode?: string): Promise<FlowDataGroup>;
  getInputDataByFlow(flow: Flow): Promise<any>;
  getInputDataById(inputId: string): Promise<any>;
  getKeyValue(params: any, cacheMode?: string): Promise<FlowKeyValue>;
  insert(flow?: any, cacheMode?: string): Promise<Flow>;
  insertKeyValue(keyValue: FlowKeyValue, cacheMode?: string): Promise<FlowKeyValue>;
  transition(flow: Flow, transition: string, options?: any): Promise<any>;
  update(flow: Flow, cacheMode?: string): Promise<Flow>;
}
export const flowService: FlowService;

export class ImportService extends JsonApiService {
  getExampleFiles(options?: any): Promise<any[]>;
}
export const importService: ImportService;

export class JsonApiService {
  delete(obj: any, options?: any): Promise<any>;
  get(params?: any, options?: any): (Promise<any>);
  getChildren(parent: any, child?: any, options?: any): (Promise<any>);
  insert(obj?: any, options?: any): Promise<any>;
  update(obj: any, options?: any): Promise<any>;
}

export class LanguageService extends JsonApiService{
  delete(obj: any, cacheMode?: string): Promise<any>;
  get(params?: any, cacheMode?: string): Promise<Language[] | Language>;
  getLanguages(getTranslations?: boolean): Promise<{ [id: string] : Language } | Language>;
  getTranslations(iso: string): Promise<any>;
  insert(obj?: any, cacheMode?: string): Promise<Language>;
  update(obj: any, cacheMode?: string): Promise<Language>;
}
export const languageService: LanguageService;

export class LocationService {
  delete(obj: any, cacheMode?: string): Promise<any>;
  get(params?: any, cacheMode?: string): Promise<any>;
  getCurrent(): Promise<any>;
  getTimezones(params?: any, cacheMode?: string): Promise<Timezone[]>;
  insert(location?: any, cacheMode?: string): Promise<any>;
  insertV2(location: any): Promise<any>;
  insertCurrent(): Promise<any>;
  update(location: Location, cacheMode?: string): Promise<any>;
}
export const locationService: LocationService;

export class MessageService {
  getConversations(): Promise<any>;
  getMessages(id: string): Promise<any>;
  sendMessage(message: any): Promise<any>;
}
export const messageService: MessageService;

export class PlatformMenuService extends JsonApiService {
  delete(platformMenu: PlatformMenu, cacheMode?: string): Promise<any>;
  get(params?: any, cacheMode?: string): (Promise<PlatformMenu[] | PlatformMenu>);
  insert(platformMenu?: any, cacheMode?: string): Promise<PlatformMenu>;
  update(platformMenu: PlatformMenu, cacheMode?: string): Promise<PlatformMenu>;
}
export const platformMenuService: PlatformMenuService;

export class ProjectContentService {
  delete(version: string, ownerType: string, owner?: number, type?: string, id?: number): Promise<any>;
  deleteProject(project: Content, type?: string, id?: number): Promise<any>;

  get(version: string, ownerType: string, owner?: number, contentType?: string, contentId?: number): Promise<any>;
  getProject(project: Content, contentType?: string, contentId?: number): Promise<any>;

  insert(version: string, ownerType: string, owner: number, contentType: string, data: any): Promise<any>;
  insertProject(project: Content, type?: string, data?: any): Promise<any>;

  update(version: string, ownerType: string, owner: number, contentType: string, contentId: number, data: any): Promise<any>;
  updateProject(project: Content, type?: string, id?: number, data?: any): Promise<any>;
}
export const projectContentService: ProjectContentService;

export class RelatedUserService {
  delete(relatedUser: RelatedUser, cacheMode?: string): Promise<any>;
  get(params?: any, cacheMode?: string): (Promise<RelatedUser[] | RelatedUser>);
  getByUser(params?: any, cacheMode?: string): (Promise<RelatedUser[] | RelatedUser>);
  insert(relatedUser: RelatedUser, cacheMode?: string): Promise<RelatedUser>;
  update(relatedUser: RelatedUser, cacheMode?: string): Promise<RelatedUser>;
}
export const relatedUserService: RelatedUserService;

export class RoleService {
  delete(obj: any, cacheMode?: string): Promise<any>;
  deletePermission(permission: Permission, cacheMode?: string): Promise<any>;
  get(params?: any, cacheMode?: string): (Promise<Role[] | Role>);
  getAuthorizableResources(): Promise<[ContentType | Section | Flow]>;
  getInvitableRoles(role: Role, cacheMode?: string): (Promise<Role[] | Role>);
  getPermissions(role: Role, options?: object);
  insert(role?: Role, cacheMode?: string): Promise<Role>;
  insertPermission(permission: Permission, role: Role, authorizable: any, cacheMode?: string): Promise<Permission>;
  update(role: Role, cacheMode?: string): Promise<Role>;
}
export const roleService: RoleService;

export class RuleService {
  delete(rule: Rule, cacheMode?: string): Promise<any>;
  get(params?: any, cacheMode?: string): (Promise<Rule[] | Rule>);
  insert(rule?: any, cacheMode?: string): Promise<Rule>;
  update(rule: Rule, cacheMode?: string): Promise<Rule>;

  // old v2
  getEventsByDate(ruleId: string, date: string): Promise<any>;
  getEventsByRule(ruleId: string): Promise<any>;
  getFilteredRecentEvents(ruleId: string, timeUnit?: string): Promise<any>;
  getNextEvent(ruleId: string): Promise<any>;
  getRecentEvents(timeUnit?: string): Promise<any>;
  getUpcomingEvents(ruleId: string, timeUnit?: string): Promise<any>
  markEvent(ruleId: string, eventId: string, eventAction: string): Promise<any>;
  markEventByObj(ruleId: string, eventChange: any): Promise<any>;
}
export const ruleService: RuleService;

export class StatusService {
  check(obj: any, options?: any): Promise<Status>;
  getSections(resource: string, ownerType: string, owner?: string): Promise<any>;
  getStatus(type: string, object: string, ownerType: string, owner?: string): Promise<any>;
  transition(transition: string, type: string, object: any, ownerType: string, owner?: string): Promise<any>;
}
export const statusService: StatusService;

export class TemporaryIdService extends JsonApiService {
  get(tempIdentifier: TemporaryId, cacheMode?: string): Promise<TemporaryId>;
  insert(tempIdentifier: TemporaryId, cacheMode?: string): Promise<TemporaryId>;
}
export const temporaryIdService: TemporaryIdService;

export class SystemService {
  log(logEntry?: any, requestInformation?: any, deviceInformation?: any): Promise<any>;
  xAuth(json: any): Promise<any>;
}
export const systemService: SystemService;

export class UserService extends JsonApiService {
  constructor(type?);
  acceptInvitation(token: string, options?: any): Promise<User>;
  acceptInvitationPinV2(params: {pin: string|number, pin_confirmation: string|number, invitation_token: string}): Promise<any>;
  addAwardedBadge(awardedBadge: any): Promise<AwardedBadge>;
  addEntry(user: User, attributes: any, template: EntryTemplate): Promise<Entry[] | Entry>;
  addSchedule(schedule?: Schedule): Promise<Schedule>;
  confirm(token: string, cacheMode?: string): Promise<any>;
  delete(user: User, cacheMode?: string): Promise<any>;
  deleteRelated(id: number): Promise<any>;
  deleteProfileAttribute(id: number): Promise<any>;
  deleteCurrent(): Promise<any>;
  disableCurrent(): Promise<any>;
  disable(id: number): Promise<any>;
  emailAccountName(emailAddress: string): Promise<any>;
  emailPassword(emailAddress: string): Promise<any>;
  enableUser(id: number): Promise<any>;
  get(params?: any, cacheMode?: string): Promise<User[] | User>;
  getAllProfileAttributes(apiKey: string): Promise<any>;
  getAwardedBadges(user: User, awardedBadge?: any, cacheMode?: string): Promise<AwardedBadge[] | AwardedBadge>;
  getDataGroup(dataGroup: any, user?: any, cacheMode?: string): Promise<FlowDataGroup>;
  getSchedules(user?: any, cacheMode?: string): Promise<Schedule[]>;
  getSites(user?: any, cacheMode?: string): Promise<Site[]>;
  getEntries(user: User, filter?: any, cacheMode?: string): Promise<Entry[] | Entry>;
  getNotifications(user?: User, cacheMode?: string): Promise<Notification[]>;
  getProfile(user?: User, cacheMode?: string): Promise<Profile>;
  getProfileV2(): Promise<Profile>;
  getProfileAttributes(includeHidden?: boolean, includeValues?: boolean): Promise<any>;
  getRegistrationStatus(attributes: any): Promise<string>;
  getRelatedMobileUsers(): Promise<any>;
  getSession(token?: string, cacheMode?: string): Promise<any>;
  insert(user: User, cacheMode?: string): Promise<User>;
  insertProfileAttribute(key: string, val: any): Promise<any>;
  invite(user: User, attributes?: any, relationships?: any): Promise<User>;
  register(attributes: any, device?: Device): Promise<User>;
  removeAwardedBadge(awardedBadge?: any, cacheMode?: string): Promise<any>;
  requestPasswordReset(attributes: any): Promise<any>;
  resendInvite(id: number, options: any): Promise<any>;
  resetPassword(attributes: { reset_password_token: string, password: string, email?: string }, device?: Device): Promise<User>;
  resetPasswordSimple(newPassword: string, newPasswordConf: string, oldPassword: string): Promise<any>;
  revokeConfirmation(user: User, cacheMode?: string): Promise<any>;
  revokeConsent(user: User, cacheMode?: string): Promise<any>;
  sendConfirmation(email: string): Promise<any>;
  signIn(attributes: any, device?: Device, cacheMode?: string): Promise<Session>;
  setPin(json: any): Promise<any>;
  signOut(): Promise<any>;
  signUp(udid: string, platform: string, pushId: string, appVersion: string, username: string, pw: string, profile: Profile): Promise<any>;
  unlockUser(email: string, unlockToken: string): Promise<any>;
  update(user: User, cacheMode?: string): Promise<User>;
  updateAwardedBadge(awardedBadge?: any, cacheMode?: string): Promise<AwardedBadge[] | AwardedBadge>;
  saveProfile(params: Profile | User | { profile: any, user?: { id: number, type: string } }, options?: any): Promise<Profile>;
  updateProfileAttribute(profile: any, id: number): Promise<any>;
  updateSchedule(schedule: Schedule, cacheMode?: string): Promise<Schedule>;
}
export const userService: UserService;
export const mobileUserService: UserService;


// Utility Classes
export const ArrayUtility: {
  union(arr1: any[], arr2: any[], eqFn: Function): any[];
  unique(arr: any[], eqFn: Function): any[];
  arrayToObject(_array: any[], options?: any): any;
}

export const ClassUtility: {
  aggregate(baseClass, ...mixins);
  camelToSnake(key: string): string;
  kebabToCamel(key: string): string;
  kebabToSnake(key: string): string;
  snakeToCamel(key: string): string;
  stringToSnake(key: string): string;
  fieldsToSnake(obj: Object): Object;
}

export const ConditionalUtility: {
  operators: { [ id: string ] : ( a: any, b: any ) => boolean };
  evaluate: (a: any, operator: string, b: any) => boolean;
}

export class StorageUtility {
  constructor(enforcer);
  static instance: StorageUtility;
  data: any;
  clear(type?: string, id?: number): Promise<any>;
  get(type: string, options?: Object): Promise<any>;
  has(type: string, id?: number): Promise<any>;
  set(type: string, value: any, options?: Object): Promise<any>;
}

export const ValidationUtility: {
  hasToken: (msg?: string) => string;
  isDate: (date: any, msg?: string) => string;
  isRequired: (p: any, msg?: string) => string;
  isA: (p: any, type: string, msg?: string) => string;
  hasAttribute: (p: any, attributeName: string, msg?: string) => string;
  isValid: (booleanStatement: boolean, msg?: string) => string;
  validate: (module: string, ...results: Object[]) => string;
}

export class HelperFactory {
  static map: any;
  get(json: any, type?: string, meta?: any): any;
  getClass(json: any, type?: string): any;
}
export const helperFactory: HelperFactory;

export class ServiceFactory {
  static map: any;
  static options: any;
  get(type: string): any;
  getObjectName(type: string): any;
}
export const serviceFactory: ServiceFactory;
